package io.takima.agencymanagement;

import io.takima.agencymanagement.model.Airport;
import io.takima.agencymanagement.model.Discount;
import io.takima.agencymanagement.model.RestaurentOffer;
import io.takima.agencymanagement.model.PercentDiscount;
import io.takima.agencymanagement.model.Travel;
import io.takima.agencymanagement.model.User;
import io.takima.agencymanagement.service.TravelService;
import io.takima.agencymanagement.utils.AirportManager;

import java.time.Duration;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.ArrayDeque;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Queue;
import java.util.Set;

public class Main {

    public static void main(String[] args) {

        TravelService travelService = new TravelService();
        //Update Main
        System.out.printf("Agency App \n");

        User user1 = new User(0, "John", "Doe", "john.doe@gmail.com");
        User user2 = new User(1, "John", "Doe", "john.doe@gmail.com");
        User user3 = new User(2, "John", "Doe", "john.doe@gmail.com");
        User user4 = new User(3, "John", "Doe", "john.doe@gmail.com");
        User user5 = new User(4, "John", "Doe", "john.doe@gmail.com");
        User user6 = new User(5, "Mat", "Doe", "mat.doe@gmail.com");

        //Add 50% discount
        List<Discount> discounts = new ArrayList<>();
        discounts.add(new PercentDiscount(50));

        //Add participant set
        Set<User> users = Set.of(user1, user2, user3, user4, user5);
        Set<User> participants = new HashSet<>(users);

        // Bug without overriding equals and hashCode
        User sameUser1 = new User(0, "Same", "User", "sameuser@gmail.com");
        User sameUser2 = new User(0, "Same", "User", "sameuser@gmail.com");

        Set<User> sameUsers = new HashSet<>();
        try {
            sameUsers = Set.of(sameUser1, sameUser2);
        } catch (IllegalArgumentException e) {
            System.out.println(e.getMessage());
        }
        Travel travelWithBug = new Travel(0L, null, Instant.now(), Instant.now().plusSeconds(1),
                null, null, null, 0, 0, null,
                sameUsers, null);

        System.out.println(travelWithBug.getParticipants());

        //Add waitList
        Queue<User> waitList = new ArrayDeque<>();
        waitList.add(user6);

        // Log RestaurentOffer details
        RestaurentOffer offer = new RestaurentOffer(
                0L,
                4,
                "description",
                2,
                Instant.now().plusSeconds(10),
                2,
                6
        );
        System.out.println(offer.getDetails());

        //Create firstTravel
        Travel firstTravel = new Travel(0L, "to Japan", Instant.now(), Instant.now().plusSeconds(86400), Airport.PARIS_CHARLES_DE_GAULLE, Airport.TOKYO_HANEDA, "Tokyo", 5, 2000, discounts, participants, waitList);

        //Apply discount
        applyDiscounts(firstTravel);

        //Log user information
        System.out.println(user1);
        //Log travel information
        System.out.println(firstTravel);

        //Subscribe user using TravelService and get an exception
        try {
            travelService.subscribe(user1, firstTravel);
        } catch (Exception e) {
            System.out.println(e.getMessage());
        }

        System.out.println("Utilisateurs en file d'attente : " + firstTravel.getWaitList().element());

        //Unsubscribe a participant then check the first one in the waitList is subscribed
        travelService.unsubscribe(user1, firstTravel);

        for (User participant : firstTravel.getParticipants()) {
            System.out.println(participant);
        }

        //Log CDG airport timeZone
        System.out.println(AirportManager.timeZone.get(Airport.PARIS_CHARLES_DE_GAULLE));

        //Use computeTravelDuration
        String dateStr = "26/04/2023, 12:10";
        String dateStr2 = "27/04/2023, 06:36";

        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("dd/MM/yyyy, HH:mm");
        LocalDateTime localDateTime1 = LocalDateTime.parse(dateStr, formatter);
        LocalDateTime localDateTime2 = LocalDateTime.parse(dateStr2, formatter);


        Instant instant1 = localDateTime1.atZone(ZoneId.of(AirportManager.timeZone.get(Airport.PARIS_CHARLES_DE_GAULLE))).toInstant();
        Instant instant2 = localDateTime2.atZone(ZoneId.of(AirportManager.timeZone.get(Airport.TOKYO_HANEDA))).toInstant();

        firstTravel.setDepartureDate(instant1);
        firstTravel.setArrivalDate(instant2);

        System.out.println(travelService.computeTravelDuration(firstTravel));

        //Use adjustDates
        travelService.adjustDates(Duration.ofHours(5), firstTravel);
        //Log it in the appropriate zoneTime
        System.out.println(firstTravel.getDepartureDate().atZone(ZoneId.of(AirportManager.timeZone.get(firstTravel.getDepartureAirport()))));

        //Find available ones
        List<Travel> availableTravels = travelService.findAvailableForDestination("Paris");

    }

    //Before services
    public static void applyDiscounts(Travel travel) {
        if (travel.getDiscounts().isEmpty()) {
            System.out.println("Aucune réduction n'est applicable sur ce voyage !");
        } else {
            for (Discount discount : travel.getDiscounts()) {
                double discountedPrice = discount.apply(travel.getPrice());
                System.out.println("Prix après réduction : " + discountedPrice);
            }
        }
    }

    public static boolean subscribe(User user, Travel travel) {

        if (travel.getParticipants().size() < travel.getCapacity()) {
            travel.getParticipants().add(user);
            return true;
        }
        travel.getWaitList().add(user);
        return false;
    }

    public void unsubscribe(User user, Travel travel) {

        travel.getParticipants().remove(user);

        if (!travel.getWaitList().isEmpty()) {
            travel.getParticipants().add(travel.getWaitList().poll());
        }
    }
}