package io.takima.agencymanagement.model;

import java.time.Instant;
import java.util.List;
import java.util.Queue;
import java.util.Set;

public class Travel {

    private final long id;
    private final String name;
    private Instant departureDate;
    private Instant arrivalDate;
    private final Airport departureAirport;
    private final Airport arrivalAirport;
    private final String destination;
    private final int capacity;
    private final double price;
    private final List<Discount> discounts;

    private final Set<User> participants;

    private final Queue<User> waitList;

    public Travel(long id, String name, Instant departureDate, Instant arrivalDate, Airport departureAirport, Airport arrivalAirport, String destination, int capacity, double price, List<Discount> discounts, Set<User> participants, Queue<User> waitList) {
        if(departureDate.isAfter(arrivalDate)){
            throw new IllegalArgumentException("Date de départ et après la date d'arrivée");
        }

        this.id = id;
        this.name = name;
        this.departureDate =departureDate;
        this.arrivalDate = arrivalDate;
        this.departureAirport = departureAirport;
        this.arrivalAirport = arrivalAirport;
        this.destination = destination;
        this.capacity = capacity;
        this.price = price;
        this.discounts = discounts;
        this.participants = participants;
        this.waitList = waitList;
    }

    public long getId() {
        return id;
    }

    public String getName() {
        return name;
    }

    public Instant getDepartureDate() {
        return departureDate;
    }

    public void setDepartureDate(Instant departureDate) {
        this.departureDate = departureDate;
    }

    public Instant getArrivalDate() {
        return arrivalDate;
    }

    public void setArrivalDate(Instant arrivalDate) {
        this.arrivalDate = arrivalDate;
    }

    public Airport getDepartureAirport() {
        return departureAirport;
    }
    public Airport getArrivalAirport() {
        return arrivalAirport;
    }
    public String getDestination() {
        return destination;
    }
    public int getCapacity() {
        return capacity;
    }
    public double getPrice() {
        return price;
    }
    public List<Discount> getDiscounts() {
        return discounts;
    }

    public Set<User> getParticipants() {
        return participants;
    }

    public Queue<User> getWaitList() {
        return waitList;
    }

    @Override
    public String toString() {

        StringBuilder stringBuilder = new StringBuilder();
        stringBuilder.append("Le voyage '")
                .append(name)
                .append("' à destination de ")
                .append(destination)
                .append(" a ")
                .append(participants.size())
                .append(" voyageurs inscrits qui sont respectivement:\n");

        for (User participant : participants) {
            stringBuilder.append("- ").append(participant.getFirstName()).append(participant.getLastName()).append("\n");
        }

        return stringBuilder.toString();
    }

    public boolean hasAvailableCapacity(Set<User> participants, int capacity) {
        return participants.size() < capacity;
    }
}
